unit UFormMain;

{$MODE Delphi}

interface

uses
  SysUtils, Variants, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls,
  UCompiler, Registry, ComCtrls, ExtCtrls, SynEdit, SynHighlighterPas,
  UProject, FileUtil, Menus, ActnList,
  UCoolTranslator, UFormTargetCode, UFormCodeTree, URegistry, ULastOpenedList, UApplicationInfo,
  UDebugLog;

const
  RegistryRootKey = HKEY_CURRENT_USER;

type

  { TCustomCompiler }

  TCustomCompiler = class(TCompiler)
    procedure LoadFromRegistry(Root: HKEY; Key: string);
    procedure SaveToRegistry(Root: HKEY; Key: string);
  end;

  { TFormMain }

  TFormMain = class(TForm)
    ABuild: TAction;
    AHelp: TAction;
    ARunToCursor: TAction;
    AStepOut: TAction;
    AStepIn: TAction;
    AStepOver: TAction;
    AReset: TAction;
    APause: TAction;
    AStop: TAction;
    AViewTargets: TAction;
    AViewCompiledSoruce: TAction;
    AViewCodeTree: TAction;
    AViewOptions: TAction;
    AViewMessages: TAction;
    AViewSourceEditor: TAction;
    AViewObjectInspector: TAction;
    AViewProject: TAction;
    AHomepage: TAction;
    ARun: TAction;
    AAbout: TAction;
    AExit: TAction;
    AProjectNew: TAction;
    AProjectOpen: TAction;
    AProjectSave: TAction;
    AProjectSaveAs: TAction;
    AProjectClose: TAction;
    ActionList1: TActionList;
    CoolTranslator1: TCoolTranslator;
    DebugLog1: TDebugLog;
    ImageList1: TImageList;
    MainMenu: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem10: TMenuItem;
    MenuItem11: TMenuItem;
    MenuItem12: TMenuItem;
    MenuItem13: TMenuItem;
    MenuItem14: TMenuItem;
    MenuItem15: TMenuItem;
    MenuItem16: TMenuItem;
    MenuItem17: TMenuItem;
    MenuItem18: TMenuItem;
    MenuItem19: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem20: TMenuItem;
    MenuItem21: TMenuItem;
    MenuItem22: TMenuItem;
    MenuItem23: TMenuItem;
    MenuItem24: TMenuItem;
    MenuItem25: TMenuItem;
    MenuItem26: TMenuItem;
    MenuItem27: TMenuItem;
    MenuItem28: TMenuItem;
    MenuItem29: TMenuItem;
    MenuItem30: TMenuItem;
    MenuItem31: TMenuItem;
    MenuItem32: TMenuItem;
    MenuItem33: TMenuItem;
    MenuItemProducer: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem8: TMenuItem;
    MenuItem9: TMenuItem;
    MenuItemOpenRecent: TMenuItem;
    OpenDialog1: TOpenDialog;
    PageControlMain: TPageControl;
    PageControlRight: TPageControl;
    PageControlBottom: TPageControl;
    SaveDialog1: TSaveDialog;
    Splitter1: TSplitter;
    Splitter2: TSplitter;
    Splitter3: TSplitter;
    TabSheetProject: TTabSheet;
    TabSheetCodeTree: TTabSheet;
    TabSheetMessages: TTabSheet;
    TabSheetBreakpoints: TTabSheet;
    TabSheetTargetProject: TTabSheet;
    TabSheetSource: TTabSheet;
    TabSheetTarget: TTabSheet;
    ToolBar1: TToolBar;
    ToolButton1: TToolButton;
    ToolButton10: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    ToolButton9: TToolButton;
    procedure AAboutExecute(Sender: TObject);
    procedure AExitExecute(Sender: TObject);
    procedure AHomepageExecute(Sender: TObject);
    procedure APauseExecute(Sender: TObject);
    procedure AProjectCloseExecute(Sender: TObject);
    procedure AProjectNewExecute(Sender: TObject);
    procedure AProjectOpenExecute(Sender: TObject);
    procedure AProjectSaveAsExecute(Sender: TObject);
    procedure AProjectSaveExecute(Sender: TObject);
    procedure ABuildExecute(Sender: TObject);
    procedure AResetExecute(Sender: TObject);
    procedure ARunExecute(Sender: TObject);
    procedure ARunToCursorExecute(Sender: TObject);
    procedure AStepInExecute(Sender: TObject);
    procedure AStepOutExecute(Sender: TObject);
    procedure AStepOverExecute(Sender: TObject);
    procedure AStopExecute(Sender: TObject);
    procedure AViewCodeTreeExecute(Sender: TObject);
    procedure AViewCompiledSoruceExecute(Sender: TObject);
    procedure AViewMessagesExecute(Sender: TObject);
    procedure AViewOptionsExecute(Sender: TObject);
    procedure AViewTargetsExecute(Sender: TObject);
    procedure AViewProjectExecute(Sender: TObject);
    procedure AViewSourceEditorExecute(Sender: TObject);
    procedure ComboBox1Change(Sender: TObject);
    procedure ComboBoxProducerChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    LastOpenedFiles: TLastOpenedList;
    procedure ProjectTemplatesInit;
    procedure CompilerDebugLog(Text: string);
    procedure MenuItemTargetClick(Sender: TObject);
    procedure OpenRecentClick(Sender: TObject);
    procedure DockInit;
    procedure LoadFromRegistry(Root: HKEY; Key: string);
    procedure SaveToRegistry(Root: HKEY; Key: string);
    procedure ProjectOpen(FileName: string);
    procedure UpdateMenu;
    procedure UpdateToolbar;
  public
    ReopenLastOpenedFile: Boolean;
    LogParsing: Boolean;
    Project: TProject;
    ProjectTemplates: TProjectTemplateList;
    TargetProject: TProject;
    Compiler: TCustomCompiler;
    function GetSource(Name: string; var SourceCode: string): Boolean;
    procedure UpdateInterface;
  end;

var
  FormMain: TFormMain;


implementation

{$R *.lfm}

uses
  UFormMessages, UFormSourceCode, UFormProject, UCommon, UFormAbout, UFormOptions,
  UFormTargets, UTarget, UAnalyzer, UProducer, UExecutor, UFormProjectNew,
  UProjectTemplates, UFormTargetProject;

resourcestring
  SNewProject = 'New project';

{ TCustomCompiler }

procedure TCustomCompiler.LoadFromRegistry(Root: HKEY; Key: string);
var
  I: Integer;
begin
  with TRegistryEx.Create do
    try
      RootKey := Root;
      for I := 0 to Targets.Count - 1 do
      with TTarget(Targets[I]) do begin
        OpenKey(Key + '\Targets\' + Name, True);
        if ValueExists('CompilerPath') then
          Producer.CompilerPath := ReadStringWithDefault('CompilerPath', Producer.CompilerPath);
      end;
      if ValueExists('SelectedTarget') then
      Target := Targets.SearchBySysName(ReadString('SelectedTarget'))
        else Target := nil;
    finally
      Free;
    end;
end;

procedure TCustomCompiler.SaveToRegistry(Root: HKEY; Key: string);
var
  I: Integer;
begin
  with TRegistryEx.Create do
    try
      RootKey := Root;
      for I := 0 to Targets.Count - 1 do
      with TTarget(Targets[I]) do begin
        OpenKey(Key + '\Targets\' + Name, True);
        if Assigned(Producer) then begin
          if Producer.CompilerPath <> '' then WriteString('CompilerPath', Producer.CompilerPath)
            else DeleteValue('CompilerPath');
        end;
      end;
      if Assigned(Target) then WriteString('SelectedTarget', Target.SysName)
        else DeleteValue('SelectedTarget');
    finally
      Free;
    end;
end;

{ TFormMain }

procedure TFormMain.ABuildExecute(Sender: TObject);
begin
  FormSourceCode.Save;

  // Compile project file
    Compiler.Init;

    if LogParsing then
      Compiler.Analyzer.OnDebugLog := CompilerDebugLog
      else Compiler.Analyzer.OnDebugLog := nil;

    if Project.Files.Count > 0 then
    with TProjectFile(Project.Files[0]) do begin
      Compiler.SourceFiles.Clear;
      Project.Files.DumpFileList(Compiler.SourceFiles);
      Compiler.MainSource := Project.MainSource.AbsoluteFileName;
      TargetProject.FileName := ExtractFileDir(Project.FileName) + DirectorySeparator +
        Compiler.CompiledFolder + DirectorySeparator + Compiler.Target.Name +
        DirectorySeparator + 'Target.tppr';
      Compiler.TargetFolder := ExtractFileDir(TargetProject.FileName);
      Compiler.Compile;
      TargetProject.Files.LoadFromList(Compiler.TargetFiles);
      FormTargetProject.UpdateInterface;
    end;
    FormProject.TreeViewProjectChange(Self, FormProject.TreeViewProject.Selected);
    FormMessages.Reload;
end;

procedure TFormMain.AResetExecute(Sender: TObject);
begin
  Compiler.Target.Executor.Reset;
end;

procedure TFormMain.ARunExecute(Sender: TObject);
begin
  ABuildExecute(Self);
  Compiler.Target.Executor.Run;
end;

procedure TFormMain.ARunToCursorExecute(Sender: TObject);
begin
  Compiler.Target.Executor.RunToCursor(0); // determine position
end;

procedure TFormMain.AStepInExecute(Sender: TObject);
begin
  Compiler.Target.Executor.StepIn;
end;

procedure TFormMain.AStepOutExecute(Sender: TObject);
begin
  Compiler.Target.Executor.StepOut;
end;

procedure TFormMain.AStepOverExecute(Sender: TObject);
begin
  Compiler.Target.Executor.StepOver;
end;

procedure TFormMain.AStopExecute(Sender: TObject);
begin
  Compiler.Target.Executor.Stop;
end;

procedure TFormMain.AViewCodeTreeExecute(Sender: TObject);
begin
  PageControlRight.TabIndex := 1;
end;

procedure TFormMain.AViewCompiledSoruceExecute(Sender: TObject);
begin
  PageControlBottom.TabIndex := 1;
end;

procedure TFormMain.AViewMessagesExecute(Sender: TObject);
begin
  PageControlBottom.TabIndex := 0;
end;

procedure TFormMain.AViewOptionsExecute(Sender: TObject);
begin
  FormOptions.ShowModal;
end;

procedure TFormMain.AViewTargetsExecute(Sender: TObject);
begin
  FormTargets.Show;
end;

procedure TFormMain.AViewProjectExecute(Sender: TObject);
begin
  PageControlRight.TabIndex := 0;
end;

procedure TFormMain.AViewSourceEditorExecute(Sender: TObject);
begin

end;

function TFormMain.GetSource(Name: string; var SourceCode: string): Boolean;
var
  FileName: string;
  F: TFileStream;
begin
  FileName := ExtractFileDir(Project.FileName) + Name + '.pas';
  if FileExists(FileName) then
  try
    F := TFileStream.Create(FileName, fmOpenRead);
    SetLength(SourceCode, F.Size);
    if F.Size > 0 then
      F.Read(SourceCode[1], F.Size);
    Result := True;
  finally
    F.Free;
  end else Result := False;
end;

procedure TFormMain.UpdateInterface;
var
  I: Integer;
  Title: string;
begin
  Title := ApplicationInfo.Name;
  if Project.FileName <> '' then Title := Project.FileName + ' - ' + Title;
  if Project.Modified then Title := Title + ' *';
  Caption := Title;
  AProjectClose.Enabled := Project.Active;
  AProjectSave.Enabled := Project.Active and Project.Modified;
  AProjectSaveAs.Enabled := Project.Active;
  (*AProgramRun.Enabled := Project.Active and (BrainFuckInterpreter.State = rsStopped);
  AProgramPause.Enabled := Project.Active and (BrainFuckInterpreter.State = rsRunning);
  AProgramStop.Enabled := Project.Active and (BrainFuckInterpreter.State <> rsStopped);*)
  ABuild.Enabled := Project.Active and Assigned(Compiler.Target) and
    Assigned(Compiler.Target.Producer);
  APause.Enabled := Project.Active and Assigned(Compiler.Target) and
    Assigned(Compiler.Target.Executor) and (Compiler.Target.Executor.State = rsRunning);
  ARun.Enabled := Project.Active and Assigned(Compiler.Target) and
    Assigned(Compiler.Target.Executor) and ((Compiler.Target.Executor.State = rsStopped) or
    (Compiler.Target.Executor.State = rsPaused));
  AStop.Enabled := Project.Active and Assigned(Compiler.Target) and
    Assigned(Compiler.Target.Executor) and ((Compiler.Target.Executor.State = rsRunning) or
    (Compiler.Target.Executor.State = rsPaused));
  AStepIn.Enabled := Project.Active and Assigned(Compiler.Target) and
    Assigned(Compiler.Target.Executor) and ((Compiler.Target.Executor.State = rsRunning) or
    (Compiler.Target.Executor.State = rsPaused));
  AStepOut.Enabled := AStepIn.Enabled;
  AStepOver.Enabled := AStepIn.Enabled;
  ARunToCursor.Enabled := AStepIn.Enabled;

  UpdateMenu;
  UpdateToolbar;
  FormSourceCode.UpdateInterface;
  FormTargetCode.UpdateInterface;
  FormProject.UpdateInterface;
end;

procedure TFormMain.DockInit;
//var
  //Container1: TCDConjoinForm;
  //Container2: TCDConjoinForm;
begin
  FormSourceCode.ManualDock(TabSheetSource, nil, alClient);
  FormSourceCode.Align := alClient;
  FormSourceCode.Show;
  FormMessages.ManualDock(TabSheetMessages, nil, alClient);
  FormMessages.Align := alClient;
  FormMessages.Show;
  FormProject.ManualDock(TabSheetProject, nil, alClient);
  FormProject.Align := alClient;
  FormProject.Show;
  FormTargetProject.ManualDock(TabSheetTargetProject, nil, alClient);
  FormTargetProject.Align := alClient;
  FormTargetProject.Show;
  FormCodeTree.ManualDock(TabSheetCodeTree, nil, alClient);
  FormCodeTree.Align := alClient;
  FormCodeTree.Show;
  FormTargetCode.ManualDock(TabSheetTarget, nil, alClient);
  FormTargetCode.Align := alClient;
  FormTargetCode.Show;
  PageControlRight.TabIndex := 0;
  PageControlBottom.TabIndex := 0;
  PageControlMain.TabIndex := 0;

  (*ProjectManager.ManualDock(DockPanel, nil, alLeft);
  ProjectManager.Show;

  Container1 := TCDManager(DockPanel.DockManager).CreateConjoinForm;
  Container1.Caption := 'Tools';
  Container1.Name := 'Tools';
  Container1.ManualDock(DockPanel, nil, alRight);
  Container1.Show;
  Container1.Parent.Parent.Width := FormMain.Width;

  CodeTreeForm.ManualDock(DockPanel, nil, alRight);
  CodeTreeForm.Show;

  Container2 := TCDManager(DockPanel.DockManager).CreateConjoinForm;
  Container2.Caption := 'Panel';
  Container2.Name := 'Tools2';
  Container2.ManualDock(Container1, nil, alRight);
  Container2.Show;

  MessagesForm.ManualDock(Container1, nil, alBottom);
  MessagesForm.Show;

  CodeForm.ManualDock(Container2, nil, alTop);
  CodeForm.Show;


  //CompiledForm.ManualDock(Container2.Panel, nil, alTop);
  //CompiledForm.Show;

  //TCoolDockManager(Container2.Panel.DockManager).DockStyle := dsTabs;

  Container1.Parent.Parent.Width := FormMain.Width - ProjectManager.Width - 200;
  Container2.Parent.Parent.Height := FormMain.Height;*)
end;

procedure TFormMain.LoadFromRegistry(Root: HKEY; Key: string);
begin
  with TRegistryEx.Create do
    try
      RootKey := Root;
      OpenKey(Key, True);
      ReopenLastOpenedFile := ReadBoolWithDefault('ReopenLastOpenedFile', True);
      Compiler.Target := Compiler.Targets.SearchBySysName(ReadStringWithDefault('TargetName', ''));
      LogParsing := ReadBoolWithDefault('LogParsing', False);
      if ValueExists('LanguageCode') then
        CoolTranslator1.Language := CoolTranslator1.Languages.SearchByCode(ReadStringWithDefault('LanguageCode', ''))
        else CoolTranslator1.Language := CoolTranslator1.Languages.SearchByCode('');
      PageControlRight.Width := ReadIntegerWithDefault('RightPanelWidth', 120);
      PageControlBottom.Height := ReadIntegerWithDefault('BottomPanelHeight', 100);
    finally
      Free;
    end;
  LastOpenedFiles.LoadFromRegistry(RegistryRootKey, Key + '\LastOpenedFiles');
  Compiler.LoadFromRegistry(RegistryRootKey, Key + '\Compiler');
end;

procedure TFormMain.SaveToRegistry(Root: HKEY; Key: string);
begin
  with TRegistryEx.Create do
    try
      RootKey := Root;
      OpenKey(Key, True);
      WriteBool('ReopenLastOpenedFile', ReopenLastOpenedFile);
      if Assigned(Compiler.Target) then
        WriteString('TargetName', Compiler.Target.SysName)
        else DeleteValue('TargetName');
      WriteBool('LogParsing', LogParsing);
      if Assigned(CoolTranslator1.Language) and (CoolTranslator1.Language.Code <> '') then
        WriteString('LanguageCode', CoolTranslator1.Language.Code)
        else DeleteValue('LanguageCode');
      WriteInteger('RightPanelWidth', PageControlRight.Width);
      WriteInteger('BottomPanelHeight', PageControlBottom.Height);
    finally
      Free;
    end;
  LastOpenedFiles.SaveToRegistry(RegistryRootKey, Key + '\LastOpenedFiles');
  LastOpenedFiles.ReloadMenu;
  Compiler.SaveToRegistry(RegistryRootKey, Key + '\Compiler');
end;

procedure TFormMain.ProjectOpen(FileName: string);
begin
  Project.Active := True;
  Project.LoadFromFile(FileName);
  LastOpenedFiles.AddItem(FileName);
  LastOpenedFiles.ReloadMenu;
  UpdateInterface;
end;

procedure TFormMain.MenuItemTargetClick(Sender: TObject);
begin
  with TMenuItem(Sender) do begin
    Compiler.Target := TTarget(Compiler.Targets[MenuIndex]);
    UpdateInterface;
  end;
end;

procedure TFormMain.UpdateMenu;
var
  I: Integer;
  NewMenuItem: TMenuItem;
begin
  MenuItemProducer.Clear;
  for I := 0 to Compiler.Targets.Count - 1 do begin
    NewMenuItem := TMenuItem.Create(MenuItemProducer);
    NewMenuItem.Caption := TTarget(Compiler.Targets[I]).Name;
    NewMenuItem.OnClick := MenuItemTargetClick;
    if Assigned(Compiler.Target) and
      (TTarget(Compiler.Targets[I]).SysName = Compiler.Target.SysName) then NewMenuItem.Checked := True;
    MenuItemProducer.Add(NewMenuItem);
  end;
end;

procedure TFormMain.UpdateToolbar;
var
  I: Integer;
begin
  for I := 0 to ToolBar1.ButtonCount - 1 do
    TToolButton(ToolBar1.Buttons[I]).Hint := TToolButton(ToolBar1.Buttons[I]).Caption;
end;

procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  AProjectClose.Execute;
  SaveToRegistry(RegistryRootKey, ApplicationInfo.RegistryKey);
end;

procedure TFormMain.FormCreate(Sender: TObject);
var
  I: Integer;
begin
  DeleteFile(DebugLog1.FileName);
  Compiler := TCustomCompiler.Create;
  Project := TProject.Create;
  ProjectTemplates := TProjectTemplateList.Create;
  TargetProject := TProject.Create;
  LastOpenedFiles := TLastOpenedList.Create;
  LastOpenedFiles.MenuItem := MenuItemOpenRecent;
  LastOpenedFiles.ClickAction := OpenRecentClick;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  ProjectTemplates.Free;
  LastOpenedFiles.Free;
  TargetProject.Free;
  Project.Free;
  Compiler.Free;
end;

procedure TFormMain.ProjectTemplatesInit;
begin
  ProjectTemplates.AddTemplate(TProjectTemplateConsoleApp.Create);
  ProjectTemplates.AddTemplate(TProjectTemplateGUIApp.Create);
  ProjectTemplates.AddTemplate(TProjectTemplatePackage.Create);
  ProjectTemplates.AddTemplate(TProjectTemplateUnit.Create);
end;

procedure TFormMain.CompilerDebugLog(Text: string);
begin
  DebugLog1.Add('', Text);
end;

procedure TFormMain.OpenRecentClick(Sender: TObject);
begin
  if Sender is TMenuItem then
    ProjectOpen(StringReplace(TMenuItem(Sender).Caption, '&', '', [rfReplaceAll]));
end;

procedure TFormMain.FormShow(Sender: TObject);
var
  NewProjectFile: TProjectFile;
begin
  LoadFromRegistry(RegistryRootKey, ApplicationInfo.RegistryKey);
  DockInit;
  ProjectTemplatesInit;

  if ReopenLastOpenedFile and (LastOpenedFiles.Count > 0) then begin
    ProjectOpen(LastOpenedFiles[0]);
  end;

  WindowState := wsMaximized;
end;

procedure TFormMain.ComboBox1Change(Sender: TObject);
begin
  ABuildExecute(Self);
end;

procedure TFormMain.AProjectOpenExecute(Sender: TObject);
begin
  if OpenDialog1.Execute then begin
    ProjectOpen(OpenDialog1.FileName);
  end;
end;

procedure TFormMain.AProjectCloseExecute(Sender: TObject);
begin
  //if Project.Modified then ;
  Project.Active := False;
  UpdateInterface;
end;

procedure TFormMain.AExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.AAboutExecute(Sender: TObject);
begin
  FormAbout.ShowModal;
end;

procedure TFormMain.AHomepageExecute(Sender: TObject);
begin
  OpenWebPage(ApplicationInfo.HomePage);
end;

procedure TFormMain.APauseExecute(Sender: TObject);
begin
  Compiler.Target.Executor.Pause;
end;

procedure TFormMain.AProjectNewExecute(Sender: TObject);
var
  NewFile: TProjectFile;
begin
  if FormProjectNew.ShowModal = mrOk then begin
    if Assigned(FormProjectNew.ListView1.Selected) then
      TProjectTemplate(FormProjectNew.ListView1.Selected.Data).InitProject(Project);
  end;
  UpdateInterface;
end;

procedure TFormMain.AProjectSaveAsExecute(Sender: TObject);
begin
  if SaveDialog1.Execute then begin
    Project.Active := True;
    Project.SaveToFile(SaveDialog1.FileName);
    FormSourceCode.Save;
    Project.Save;
    UpdateInterface;
    LastOpenedFiles.AddItem(SaveDialog1.FileName);
    LastOpenedFiles.ReloadMenu;
  end;
end;

procedure TFormMain.AProjectSaveExecute(Sender: TObject);
begin
  FormSourceCode.Save;
  if not FileExistsUTF8(Project.FileName) then AProjectSaveAs.Execute
    else Project.SaveToFile(Project.FileName);
end;

procedure TFormMain.ComboBoxProducerChange(Sender: TObject);
begin
  ABuildExecute(Self);
end;

end.
